// Time-stamp: </wrf/c/connect/comp2pgm.cc, Tue, 12 Nov 2002, 12:34:06 EST, wrf@benvolio.ecse.rpi.edu>

// Usage:
// comp2pgm x y z nrows < compsfile > pgmfile

// Convert subsquare (x,y,z) to (x,y+n-1,z+n-l) of the components file
// to a pgm file, with random colors in 1 .. 255.


#include <iostream>
#include <time.h>
#include <stdlib.h>

using namespace std;

void die(const char *msg)
{	std::cerr << "ERROR: " << msg << endl;
 exit(1);
}

main(const int argc, const char **const argv) 
{
  char *now;
  time_t secs;
  short int *a;
  int i, m;
  int goodx, goody, goodz, goodn;
  int nx, ny, nz, connectivity, nvoxels, nruns, ncomps, ncomps_written;
  int ic;
  int kruns, vol, area;
  int ir;
  int x, y, z, zl, zh;
  int grey;

  secs = time(NULL);
  now = ctime(&secs);

  cerr << "\n***** COMP2PGM starting at " << now << "*****" << endl;
 

  if (argc!=5) {
    cerr << "argc=" << argc << endl;
    die("Usage: comp2pgm x y z nrows < compsfile > pgmfile");
  }
  goodx = atoi(argv[1]);
  goody = atoi(argv[2]);
  goodz = atoi(argv[3]);
  goodn = atoi(argv[4]);

  // read and ignore 1st 3 lines.
  
  cin >> nx >> ny >> nz >> connectivity >> nvoxels >> nruns >> ncomps >> ncomps_written;

  if (goody+goodn>ny) goodn = ny-goody;
  if (goodz+goodn>nz) goodn = nz-goodz;

  cerr << "COMP2PGM writing region (" << goodx << ',' << goody << ',' 
       << goodz << ") to (" << goodx << ',' << goody+goodn-1 << ',' 
       << goodz+goodn-1 << ')' << endl;

  cerr << "I see that the input has nx=" << nx << ", ny=" << ny << ", nz=" 
       << nz << ", connectivity=" << connectivity << ", nvoxels=" << nvoxels 
       << ", nruns=" << nruns << ", ncomps=" << ncomps << ", ncomps_written="
       << ncomps_written << endl;

  m = goodn * goodn;
  a = (short int *) malloc(m*sizeof(short int));
  if (a==NULL) die("malloc of a failed.");
  for(i=0;i<m; i++) 
    a[i] = 0;
  grey = 1;
  
  cout << "P2" << endl;
  cout << goodn << ' ' << goodn << endl;
  cout << 255 << endl;

  for (ic=0; ic<ncomps_written; ic++) {
    cin >> kruns >> vol >> area;
    if (!cin) {
      cerr << "ERROR reading kruns, vol, area failed." << endl;
      exit(1);
    }
    for (ir=0; ir<kruns; ir++) {
      cin >> x >> y >> zl >> zh;
      if (!cin) {
	cerr << "ERROR reading x, y, zl, zh failed for ir=" << ir << endl;
	exit(1);
      }
      if (x!=goodx) continue;
      if (y<goody || y>=goody+goodn) continue;
      if (y<0||y>=ny) {
	cerr << "ILLEGAL y" << endl;
	cerr << "x=" << x << ", y=" << y << ", zl=" << zl << ", zh=" << zh <<endl;
	exit(1);
      }
      if (zl<0 || zl>zh || zh>=nz) {
	cerr << "ILLEGAL zl, zh" << endl;
	cerr << "x=" << x << ", y=" << y << ", zl=" << zl << ", zh=" << zh <<endl;
	exit(1);
      }

      if (zl<goodz) zl = goodz;
      if (zh>=goodz+goodn) zh = goodz+goodn-1;
      for (z=zl; z<=zh; z++) 
	a[(y-goody)*goodn+(z-goodz)] = grey;
    }
    grey++;
    if (grey>=256) grey=1;
  }

  for (i=0; i<m; i++) {
    cout << a[i] << ' ';
    if ((i&7)==7) cout << endl;
  }

}
